import React, { useState } from "react";
import {
  useAsyncDebounce,
  useTable,
  useGlobalFilter,
  useFilters,
  usePagination
} from "react-table";
import { ChevronDoubleLeftIcon, ChevronDoubleRightIcon, ChevronLeftIcon, ChevronRightIcon } from "@heroicons/react/solid";

import "./index.css";
import PaginationDefaultButton from "../Button/Pagination/DefaultButton";
import PaginationPageButton from "../Button/Pagination/PageButton";


const DataTable = ({ columns, data, tabFilterData }) => {

  const [selectedRole, setSelectedRole] = useState('All')

  // Use the state and functions returned from useTable to build your UI
  const {
    getTableProps,
    getTableBodyProps,
    headerGroups,
    rows,

    page, // Instead of using 'rows', we'll use page,
    // which has only the rows for the active page

    // The rest of these things are super handy, too ;)
    canPreviousPage,
    canNextPage,
    pageOptions,
    pageCount,
    gotoPage,
    nextPage,
    previousPage,
    setPageSize,


    prepareRow,
    state,
    preGlobalFilteredRows,
    setGlobalFilter
  } =
    useTable({
      columns,
      data,
    },
      useGlobalFilter,
      usePagination
    );

  let row_data = page;

  if (selectedRole !== 'All') {
    row_data = page.filter(x => x.values.role_name === selectedRole || x.values.type === selectedRole);
  }


  // Render the UI for your table
  return (
    <>
      <GlobalFilter
        preGlobalFilteredRows={preGlobalFilteredRows}
        globalFilter={state.globalFilter}
        setGlobalFilter={setGlobalFilter}
        tabFilterData={tabFilterData}
        onSelectedTabFilter={(role_name) => setSelectedRole(role_name)}
        selectedRole={selectedRole}
      />
      <div className="mt-2 flex flex-col">
        <div className="-my-2 overflow-x-auto -mx-4 sm:-mx-6 lg:-mx-8">
          <div className="py-2 align-middle inline-block min-w-full sm:px-6 lg:px-8">
            <div className="shadow overflow-hidden rounded">
              <table {...getTableProps()} className="min-w-full divide-y divide-gray">
                <thead className="bg-primary">
                  {headerGroups.map((headerGroup) => (
                    <tr style={{ padding: 10 }} {...headerGroup.getHeaderGroupProps()}>
                      {headerGroup.headers.map((column) => (
                        <th className="p-2" style={{ color: 'white', textAlign: 'left' }} {...column.getHeaderProps()}>{column.render("Header")}</th>
                      ))}
                      <th style={{ color: 'white', textAlign: 'left', width: '5%' }}>Action</th>
                    </tr>
                  ))}
                </thead>
                <tbody {...getTableBodyProps()}>
                  {row_data.map((row, i) => {
                    prepareRow(row);
                    return (
                      <tr {...row.getRowProps()}>
                        {row.cells.map((cell) => {
                          return (
                            <td className="p-2"{...cell.getCellProps()}>
                              {cell.column.id === 'rowNumber' ? i + 1 : cell.render('Cell')}
                            </td>
                          );
                        })}
                        <td className="flex flex-row py-3 gap-2 px-3">
                            <button className="px-4 py-1 rounded" style={{ border: '1px solid black', fontSize: 14 }}>
                              Edit
                            </button>
                            <button className='bg-red text-white px-4 py-1 rounded' style={{ fontSize: 14}}>
                              Delete
                            </button>
                        </td>
                      </tr>
                    );
                  })}
                </tbody>
              </table>
            </div>
          </div>
        </div>
      </div>
      <div className="py-3 flex items-center justify-between">
        <div className="flex-1 flex justify-between sm:hidden">
          <PaginationDefaultButton onClick={() => previousPage()} disabled={!canPreviousPage}>Previous</PaginationDefaultButton>
          <PaginationDefaultButton onClick={() => nextPage()} disabled={!canNextPage}>Next</PaginationDefaultButton>
        </div>
        <div className="hidden sm:flex-1 sm:flex sm:items-center sm:justify-between">
          <div className="flex gap-x-2 items-baseline">
            <span className="text-sm text-gray-700">
              Page <span className="font-medium">{state.pageIndex + 1}</span> of <span className="font-medium">{pageOptions.length}</span>
            </span>
            <label>
              <span className="sr-only">Items Per Page</span>
              <select
                className="mt-1 block w-full rounded-md border-gray-300 shadow-sm focus:border-indigo-300 focus:ring focus:ring-indigo-200 focus:ring-opacity-50"
                value={state.pageSize}
                onChange={e => {
                  setPageSize(Number(e.target.value))
                }}
              >
                {[5, 10, 20].map(pageSize => (
                  <option key={pageSize} value={pageSize}>
                    Show {pageSize}
                  </option>
                ))}
              </select>
            </label>
          </div>
          <div>
            <nav className="relative z-0 inline-flex rounded-md shadow-sm -space-x-px" aria-label="Pagination">
              <PaginationPageButton
                className="rounded-l-md"
                onClick={() => gotoPage(0)}
                disabled={!canPreviousPage}
              >
                <span className="sr-only">First</span>
                <ChevronDoubleLeftIcon className="h-5 w-5 text-gray-400" aria-hidden="true" />
              </PaginationPageButton>
              <PaginationPageButton
                onClick={() => previousPage()}
                disabled={!canPreviousPage}
              >
                <span className="sr-only">Previous</span>
                <ChevronLeftIcon className="h-5 w-5 text-gray-400" aria-hidden="true" />
              </PaginationPageButton>
              <PaginationPageButton
                onClick={() => nextPage()}
                disabled={!canNextPage
                }>
                <span className="sr-only">Next</span>
                <ChevronRightIcon className="h-5 w-5 text-gray-400" aria-hidden="true" />
              </PaginationPageButton>
              <PaginationPageButton
                className="rounded-r-md"
                onClick={() => gotoPage(pageCount - 1)}
                disabled={!canNextPage}
              >
                <span className="sr-only">Last</span>
                <ChevronDoubleRightIcon className="h-5 w-5 text-gray-400" aria-hidden="true" />
              </PaginationPageButton>
            </nav>
          </div>
        </div>
      </div>
    </>
  );
}

const GlobalFilter = ({
  globalFilter,
  setGlobalFilter,
  tabFilterData,
  onSelectedTabFilter,
  selectedRole
}) => {
  const [value, setValue] = React.useState(globalFilter)
  const onChange = useAsyncDebounce(value => {
    setGlobalFilter(value || undefined)
  }, 200)

  const tabsFilter = ['All', ...tabFilterData];

  return (
    <>
      <div className="bg-gray p-3 mb-4">
        <div class="relative text-gray-600 focus-within:text-gray-400 flex flex-row">
          <span class="absolute inset-y-0  left-0 flex items-center pl-2">
            <button type="submit" class="p-1 focus:outline-none focus:shadow-outline">
              <svg fill="none" stroke="currentColor" stroke-linecap="round" stroke-linejoin="round" stroke-width="2" viewBox="0 0 24 24" class="w-4 h-4"><path d="M21 21l-6-6m2-5a7 7 0 11-14 0 7 7 0 0114 0z"></path></svg>
            </button>
          </span>
          <input
            type="search"
            name="q"
            className="py-2 text-sm text-black bg-gray-900 rounded-md pl-10 focus:outline-none focus:bg-white focus:text-gray-900"
            style={{ width: '90% ' }}
            placeholder={`Enter Keyword ...`}
            value={value}
            onChange={e => {
              setValue(e.target.value);
              onChange(e.target.value)
            }}
            autocomplete="off"
          />
          <button className='ml-4 bg-white rounded justify-center hover:bg-opacity-70 font-medium px-4 py-3 flex text-black' style={{ borderColor: 'black', borderWidth: 1 }}>
            <svg xmlns="http://www.w3.org/2000/svg" className="mr-3" style={{ alignSelf: 'center' }} fill="black" height="18" viewBox="0 -960 960 960" width="18"><path d="M440-160q-17 0-28.5-11.5T400-200v-240L161-745q-14-17-4-36t31-19h584q21 0 31 19t-4 36L560-440v240q0 17-11.5 28.5T520-160h-80Zm40-276 240-304H240l240 304Zm0 0Z" /></svg>
            Filters
          </button>
        </div>
      </div>
      <div className='flex mb-4' style={{ borderBottomWidth: 1 }}>
        {tabsFilter.map((filter) => (
          <ul>
            <a onClick={() => onSelectedTabFilter(filter)} href='#' className='mr-4 tab-filter' style={selectedRole === filter ? { borderBottom: '3px solid black', color: '#055274' } : { color: '#7fa8b9' }}>{filter}</a>
          </ul>
        ))}
      </div>
    </>
  )
}

export default DataTable;